package com.dxhy.core.thread;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.CustomsCollectVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.ICustomsCollectService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author user
 */

@SuppressWarnings("AlibabaMethodTooLong")
@Slf4j

public class CustomsSignQueryThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "customsSignQuery";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "缴款书签收查询列表";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "签收描述", "签收日期", "签收状态", "签收人", "签收方式", "来源", "管理状态", "缴款书号码", "填发日期",
        "缴款单位税号", "缴款单位名称", "税款金额", "稽核结果", "是否申请核对", "核对结果", "备注", "业务单元","凭证号","过账时间","影像号","账期","报账单号",
            "报账人","入账状态","所属组织"};

    /**
     * 缓存合计数量
     */
    private static String hjsl = "0";
    /**
     * 缓存合计金额
     */
    private static String hjje = "0";
    /**
     * 缓存合计税额
     */
    private static String hjse = "0";

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final ICustomsCollectService customsCollectService;

    private final FileService fileService;

    private final Sender sender;

    private final IFeginWebsocket feginWebsocket;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public CustomsSignQueryThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        ICustomsCollectService customsCollectService, FileService fileService, Sender sender,
        IFeginWebsocket feginWebsocket, TabulateRedisServiceImpl tabulateRedisService) {
        this.tabulateRedisService = tabulateRedisService;
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.customsCollectService = customsCollectService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
    }

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfTaxNo = prams.getString("gfTaxNo");
                String gfsh = prams.getString("gfsh");
                String companyCode = prams.getString("companyCode");
                String fillInDateBegin = prams.getString("fillInDateBegin");
                String fillInDateEnd = prams.getString("fillInDateEnd");
                String qsStatus = prams.getString("qsStatus");
                String qsType = prams.getString("qsType");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                // 参数转换为需要的形

                prams.put("dbName", dbName);
                prams.put("userAccount", userid);
                if (!StringHelper.isBlank(gfsh)) {
                    prams.put("gfsh", gfsh);
                }

                if (!StringHelper.isBlank(companyCode)) {
                    prams.put("companyCode", companyCode);
                }

                if (StringUtils.isNotBlank(fillInDateBegin)) {
                    prams.put("fillInDateBegin", DateUtils.getMorning(DateUtils.strToDate(fillInDateBegin)));
                }
                if (StringUtils.isNotBlank(fillInDateEnd)) {
                    prams.put("fillInDateEnd", DateUtils.getNight(DateUtils.strToDate(fillInDateEnd)));
                }
                String auditStatus = prams.getString("auditStatus");
                String applyCheck = prams.getString("applyCheck");
                String dataSources = prams.getString("dataSources");
                String checkStatus = prams.getString("checkStatus");
                String collectUploadStatus = prams.getString("collectUploadStatus");
                if ("99".equals(auditStatus)) {
                    prams.put("auditStatus", "");
                }
                if ("99".equals(applyCheck)) {
                    prams.put("applyCheck", "");
                }
                if ("99".equals(dataSources)) {
                    prams.put("dataSources", "");
                }
                if ("99".equals(collectUploadStatus)) {
                    prams.put("collectUploadStatus", "");
                }
                if ("99".equals(checkStatus)) {
                    prams.put("checkStatus", "");
                }
                if ("99".equals(qsStatus)) {
                    prams.put("qsStatus", "");
                }
                if ("99".equals(qsType)) {
                    prams.put("qsType", "");
                }
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(prams, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);
            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq

                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = customsCollectService.querySign(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<CustomsCollectVO> dataList = (List<CustomsCollectVO>)pageData.get("datalist");

        boolean firstPage = false;
        if (curr == default_curr) {
            hjsl = pageData.getString("number");
            hjse = pageData.getString("taxAmountSum");
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return hasNext;
        }
    }
}
