package com.dxhy.core.thread;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.ExportInvoiceRzcxVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.ICustomsCollectService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;
import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 功能描述
 *
 * @author 陈刚扬
 * @date 2020/10/30
 */

@SuppressWarnings("AlibabaMethodTooLong")
@Slf4j

public class ExportInvoiceQueryThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "exportInvoiceQueryList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "出口转内销发票查询列表";
    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "转内销证明编号", "发票代码", "发票号码", "发票类型", "发票状态", "开票日期", "购方税号", "购方名称",
        "销方税号", "销方名称", "转内销金额", "转内销税额", "转内销抵扣税额", "认证时间", "认证状态", "认证处理状态", "认证类型", "认证人", "税款所属期","凭证号","过账时间","影像号","账期","报账单号","报账人","入账状态","所属组织"};

    /**
     * 缓存合计数量
     */
    private static String hjsl = "0";
    /**
     * 缓存合计金额
     */
    private static String hjje = "0";
    /**
     * 缓存合计税额
     */
    private static String hjse = "0";

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final ICustomsCollectService customsCollectService;

    private final FileService fileService;

    private final Sender sender;

    private final IFeginWebsocket feginWebsocket;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public ExportInvoiceQueryThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        ICustomsCollectService customsCollectService, FileService fileService, Sender sender,
        IFeginWebsocket feginWebsocket, TabulateRedisServiceImpl tabulateRedisService) {
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.customsCollectService = customsCollectService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfTaxNo = prams.getString("gfTaxNo");

                String gfsh = prams.getString("gfsh");
                String companyCode = prams.getString("companyCode");
                String invoiceType = prams.getString("invoiceType");
                String invoiceSource = prams.getString("invoiceSource");
                String invoiceStatus = prams.getString("invoiceStatus");
                String rzlx = prams.getString("rzlx");
                String authStatus = prams.getString("authStatus");
                String rzhYesorno = prams.getString("rzhYesorno");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                if (!StringHelper.isBlank(gfsh)) {
                    prams.put("gfsh", gfsh);
                }

                if (!StringHelper.isBlank(companyCode)) {
                    prams.put("companyCode", companyCode);
                }

                if ("99".equals(invoiceType)) {
                    prams.put("invoiceType", "");
                }
                if ("99".equals(invoiceSource)) {
                    prams.put("invoiceSource", "");
                }

                if ("99".equals(invoiceStatus)) {
                    prams.put("invoiceStatus", "");
                }

                if ("99".equals(rzlx)) {
                    prams.put("rzlx", "");
                }

                if ("99".equals(authStatus)) {
                    prams.put("authStatus", "");
                }

                if ("99".equals(rzhYesorno)) {
                    prams.put("rzhYesorno", "");
                }

                String invoiceDateStart = prams.getString("invoiceDateStart");
                String invoiceDateEnd = prams.getString("invoiceDateEnd");
                String rzhDateStart = prams.getString("rzhDateStart");
                String rzhDateEnd = prams.getString("rzhDateEnd");
                String rzhBelongDate = prams.getString("rzhBelongDate");

                if (StringUtil.isNotEmpty(invoiceDateStart)) {
                    invoiceDateStart = DateUtils.getMorning(new SimpleDateFormat("yyyy-MM-dd").parse(invoiceDateStart));
                    prams.put("invoiceDateStart", invoiceDateStart);
                }
                if (StringUtil.isNotEmpty(invoiceDateEnd)) {
                    invoiceDateEnd = DateUtils.getNight(new SimpleDateFormat("yyyy-MM-dd").parse(invoiceDateEnd));
                    prams.put("invoiceDateEnd", invoiceDateEnd);
                }
                if (StringUtil.isNotEmpty(rzhDateStart)) {
                    rzhDateStart = DateUtils.getMorning(new SimpleDateFormat("yyyy-MM-dd").parse(rzhDateStart));
                    prams.put("rzhDateStart", rzhDateStart);
                }
                if (StringUtil.isNotEmpty(rzhDateEnd)) {
                    rzhDateEnd = DateUtils.getNight(new SimpleDateFormat("yyyy-MM-dd").parse(rzhDateEnd));
                    prams.put("rzhDateEnd", rzhDateEnd);
                }

                if (StringUtil.isNotEmpty(rzhBelongDate) && rzhBelongDate.contains("-")) {
                    Date date = new SimpleDateFormat("yyyy-MM").parse(rzhBelongDate);
                    rzhBelongDate = new SimpleDateFormat("yyyyMM").format(date);
                    prams.put("rzhBelongDate", rzhBelongDate);
                }
                boolean flag = StringUtil.isNotEmpty(rzhYesorno) && ("1".equals(rzhYesorno) || "2".equals(rzhYesorno));
                if (!flag) {
                    // 认证为1-已认证抵扣2-已认证不抵扣时，有值，其它清空认证相关查询条件
                    prams.put("rzhDateStart", "");
                    prams.put("rzhDateEnd", "");
                    prams.put("rzhBelongDate", "");
                }

                // 参数转换为需要的形式
                prams.put("dbName", dbName);

                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(prams, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);
            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);
                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = customsCollectService.queryExportInvoiceList(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<ExportInvoiceRzcxVO> dataList = (List<ExportInvoiceRzcxVO>)pageData.get("datalist");

        boolean firstPage = false;
        if (curr == default_curr) {
            hjje = pageData.getString("hjje");
            hjse = pageData.getString("hjse");
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return hasNext;
        }
    }
}
