package com.dxhy.core.thread;

import java.util.*;

import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.DictdetaServcice;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.FpcjCjhzRespVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.IFpcjService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 发票采集汇总导出线程
 *
 * @author user
 */

@SuppressWarnings("AlibabaMethodTooLong")
@Slf4j

public class FpcjhzExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private String fileNamePrefix = "invoiceCollectionList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private String title = "发票采集列表";
    /**
     * 导出excel的列名
     */
    private String[] titleColumns = {"序号", "采集日期", "购方税号", "购方名称", "采集数量合计", "金额合计"};
    private String[] titleProp;

    /**
     * 缓存合计数量
     */
    private static String hjsl = "0";
    /**
     * 缓存合计金额
     */
    private static String hjje = "0";
    /**
     * 缓存合计税额
     */
    private static String hjse = "0";

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final IFpcjService fpcjService;

    private final Sender sender;
    private final IFeginWebsocket feginWebsocket;
    private TabulateRedisServiceImpl tabulateRedisService;
    private final FileService fileService;

    public FpcjhzExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
                              IFpcjService fpcjService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
                              TabulateRedisServiceImpl tabulateRedisService) {

        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.fpcjService = fpcjService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();
            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfsh = prams.getString("gfsh");
                String cjksrq = prams.getString("cjksrq");
                String cjjsrq = prams.getString("cjjsrq");
                int ycpbz = prams.getIntValue("ycpbz");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                // 参数转换为需要的形式
                Map<String, Object> pramsMap = new HashMap<>(16);

                pramsMap.put("dbName", dbName);

                List<String> gfshList = new ArrayList<>();
                if (!StringHelper.isBlank(gfsh)) {
                    gfshList = StringHelper.stringToList(gfsh);
                    pramsMap.put("gfsh", gfshList);
                } else {
                    pramsMap.put("gfsh", gfshList);
                }
                pramsMap.put("cjksrq", DateUtils.getMorning(DateUtils.strToDate(cjksrq)));
                pramsMap.put("cjjsrq", DateUtils.getNight(DateUtils.strToDate(cjjsrq)));
                if (ycpbz == 1) {
                    pramsMap.put("ycpbz", String.valueOf(ycpbz));
                    title = "异常发票采集";
                    fileNamePrefix = "abnormalInvoiceCollectionList";
                }
                pramsMap.put("company", company);
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);

                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                String ftpFilePath = "";
                String rtnMsg = this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);
                if("ok".equals(rtnMsg)) {
                    // FTP文件存储路径
                    String[] toDay = DateUtil.today().split("-");
                    ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                            + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                    // 文件上传ftp
                    fileService.uploadFile(ftpFilePath, excelFile.toString());
                }else{
                    ftpFilePath = rtnMsg;
                }

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);
            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq

                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private String cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = fpcjService.selectByGfshAndCjrq(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<FpcjCjhzRespVO> dataList = (List<FpcjCjhzRespVO>)pageData.get("datalist");
        if(dataList.size() > 5000){
            return "数据不能超过5000条";
        }

        boolean firstPage = false;
        if (curr == default_curr) {
            hjsl = pageData.getString("hjsl");
            hjje = pageData.getString("hjje");
            hjse = pageData.getString("hjse");
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            FpcjCjhzRespVO vo = new FpcjCjhzRespVO();
            vo.setXh("合计");
            vo.setSl(hjsl);
            vo.setJe(hjje);
            vo.setSe(hjse);
            dataList.add(vo);
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
        }
        return "ok";
    }
}
