package com.dxhy.core.thread;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.entity.TDxExcelExportlog;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.SignQueryVO;
import com.dxhy.core.rabbitmq.Sender;
import com.dxhy.core.service.export.IFpqsService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 发票签收查询结果导出线程
 *
 * @author user
 */
@SuppressWarnings({"AlibabaMethodTooLong", "DuplicatedCode", "FieldCanBeLocal"})
@Slf4j

public class FpqsExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "invoiceQsList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xls";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "发票签收查询";

    /**
     * 导出excel的列名
     */
    private final String[] titleColumns = {"序号", "签收描述", "签收日期", "签收状态", "签收人", "签收方式", "发票代码", "发票号码", "发票类型", "开票日期",
        "购方名称", "购方税号", "销方名称", "销方税号", "金额", "税额", "价税合计", "扫描时间", "备注", "业务单元","凭证号","过账时间","影像号","账期","报账单号","报账人","入账状态","所属组织"};

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final IFpqsService fpqsService;

    private final FileService fileService;

    private final Sender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public FpqsExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
        IFpqsService fpqsService, FileService fileService, Sender sender, IFeginWebsocket feginWebsocket,
        TabulateRedisServiceImpl tabulateRedisService) {
        this.tabulateRedisService = tabulateRedisService;
        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.fpqsService = fpqsService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();

            String userName = exportLog.getUserName();

            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject prams = JSONObject.parseObject(condition);
                String gfsh = prams.getString("gfsh");
                String companyCode = prams.getString("companyCode");
                String businessCode = prams.getString("businessCode");
                String qsrqq = prams.getString("qsrqq");
                String qsrqz = prams.getString("qsrqz");
                String invoiceType = prams.getString("invoiceType");
                String invoiceNo = prams.getString("invoiceNo");
                if(StringUtils.isNotEmpty(invoiceNo)&&invoiceNo.length()==20){
                    invoiceNo = invoiceNo.substring(12, 20);
                }
                String qsStatus = prams.getString("qsStatus");
                String qsType = prams.getString("qsType");
                String gfshFlag = prams.getString("gfshFlag");
                String notes = prams.getString("notes");
                String xfsh = prams.getString("xfsh");
                String createDateBegin = prams.getString("createDateBegin");
                String createDateEnd = prams.getString("createDateEnd");
                String qsNameFlag = prams.getString("qsNameFlag");
                String company = prams.getString("company");
                String menuId = prams.getString("menuId");
                String gxStatus = prams.getString("gxStatus");
                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);
                // 参数转换为需要的形式
                Map<String, Object> pramsMap = new HashMap<>(21);

                pramsMap.put("userid", userid);
                pramsMap.put("dbName", dbName);
                pramsMap.put("notes", notes);
                pramsMap.put("xfsh", xfsh);
                pramsMap.put("gxStatus", gxStatus);
                pramsMap.put("businessCode", businessCode);
                pramsMap.put("gfsh", gfsh);
                pramsMap.put("companyCode", companyCode);
                pramsMap.put("gfshFlag", gfshFlag);

                pramsMap.put("settlementNo", prams.get("settlementNo"));
                pramsMap.put("settlementStatus", prams.get("settlementStatus"));
                pramsMap.put("authStatus", prams.get("authStatus"));
                pramsMap.put("voucherNumber", prams.get("voucherNumber"));

                pramsMap.put("qsrqq", DateUtils.getMorning(DateUtils.strToDate(qsrqq)));
                pramsMap.put("qsrqz", DateUtils.getNight(DateUtils.strToDate(qsrqz)));

                pramsMap.put("createDateBegin", DateUtils.getMorning(DateUtils.strToDate(createDateBegin)));
                pramsMap.put("createDateEnd", DateUtils.getNight(DateUtils.strToDate(createDateEnd)));

                if (!StringHelper.isBlank(invoiceType) && !"99".equals(invoiceType)) {
                    pramsMap.put("invoiceType", invoiceType);
                }
                if (!StringHelper.isBlank(invoiceNo)) {
                    pramsMap.put("invoiceNo", invoiceNo);
                }

                if (!StringHelper.isBlank(qsStatus)) {
                    if (!"99".equals(qsStatus)) {
                        pramsMap.put("qsStatus", String.valueOf(Integer.parseInt(qsStatus)));
                    }
                }
                if (!StringHelper.isBlank(qsType)) {
                    if (!"99".equals(qsType)) {
                        pramsMap.put("qsType", String.valueOf(Integer.parseInt(qsType)));
                    }
                }
                if (StringUtils.isNotBlank(qsNameFlag)) {
                    pramsMap.put("qsNameFlag", qsNameFlag);
                }
                pramsMap.put("company", company);
                pramsMap.put("xfmc", prams.get("xfmc"));
                pramsMap.put("invoiceSource", prams.get("invoiceSource"));
                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);
                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *             异常
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = fpqsService.selectQsfpList(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<SignQueryVO> dataList = (List<SignQueryVO>)pageData.get("datalist");

        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return false;
        }
    }
}
